using System.IO;
using System.Drawing;
using System.Drawing.Imaging;
using AdrianKousz.Util;

namespace AdrianKousz.GenieEngine
{
	public class AddBitmapCommand : BaseCommand
	{
		override public void Run()
		{
			using (var input = File.OpenRead(args[0]))
			using (var output = File.OpenWrite(args[1]))
			using (var bmpstream = File.OpenRead(args[2]))
			{
				var scn = GenieFile.Deserialize<Scenario>(input);
				var bmp = (Bitmap)Image.FromStream(bmpstream);
				var cin = scn.Cinematics;
				cin.BitmapWidth = bmp.Width;
				cin.BitmapHeight = bmp.Height;
				cin.BitmapUnknown = Scenario.ScnCinematics.ExpectedUnknownBitmapTrue;
				cin.RawBitmap = bmp.ToRawBitmap();
				GenieFile.Serialize(scn, output);
			}
		}

		override public int GetArgumentCount()
		{
			return 3;
		}

		override public string GetDescription()
		{
			return "Add a pregame bitmap";
		}

		override public string GetHelp()
		{
			return "Add a pregame bitmap to a scenario."
				+ "\nThe bitmap has to be indexed, otherwise, the game will probably crash."
				+ "\nAdditionally, a game-specific palette will be used to display the image."
				+ "\n"
				+ "\nParameters:"
				+ "\n<input scenario> <output scenario> <bitmap>"
			;
		}
	}
}
