﻿using AdrianKousz.Util;
using System.Collections.Generic;

namespace AdrianKousz.GenieEngine
{
	public class Program : IApp<string, string[]>
	{
		public static void Main(string[] args)
		{
			System.Threading.Thread.CurrentThread.Name = "main";
			Log.SetUncaughtExceptionHandler();

			if (System.Diagnostics.Debugger.IsAttached) {
				debug();
				return;
			}

			new CmdApp(new Program(), args).Run();
		}

		public string GetHeader()
		{
			return "Scenario Editor";
		}

		public IDictionary<string, ICommand<string[]>> GetCommands()
		{
			var result = new Dictionary<string, ICommand<string[]>>();
			result.Add("convert", new ConvertCommand());
			result.Add("bmp2map", new Bmp2MapCommand());
			result.Add("addbmp", new AddBitmapCommand());
			result.Add("delbmp", new DelBitmapCommand());
			result.Add("compress", new CompressCommand());
			result.Add("decompress", new DecompressCommand());
			result.Add("tojson", new ToJsonCommand());
			result.Add("fromjson", new FromJsonCommand());
			result.Add("dumpunits", new DumpunitsCommand());
			result.Add("extract", new ExtractCommand());
			result.Add("pack", new PackCommand());
			return result;
		}

		private static void debug()
		{
			var fn = "";
			using (var istream = System.IO.File.OpenRead(fn)) {
				var scn = GenieFile.Deserialize<Scenario>(istream);
				System.Diagnostics.Debugger.Break(); // Great to inspect scn
			}
		}
	}
}
