﻿using System;
using System.Collections.Generic;
using System.Text;

namespace AdrianKousz.GenieEngine
{
	public class Scenario
	{
		public const Int32 Separator = -99;
		public const Int32 NumPlayers = 16;
		public const Int32 NumPlayerSections = 9;
		public static readonly Encoding Encoding = Encoding.GetEncoding(1252);

		public const Int32 ExpectedUnknown1 = 2;
		public const Int32 ExpectedUnknown2 = 0;
		public const Int32 ExpectedUnknown3 = 1;
		public const Byte ExpectedUnknown4 = 0;
		public const Single ExpectedUnknown5 = -1;

		public String VersionString;
		public Single VersionNumber;
		public String Filename;
		public DateTime Timestamp;
		public String InstructionsUncompressed;

		public Int32 NextId;
		public Int32 PlayerCount;
		public ScnMessages Messages;
		public ScnCinematics Cinematics;
		public ScnGlobalVictory GlobalVictory;
		public ScnMap Map;

		public ScnPlayerSettings[] PlayerSettings;
		public ScnResourceCopy[] ResourceCopies;
		public IList<ScnUnit>[] Units;

		// NOTE: Raw byte arrays that do not have a parser (yet?)
		public Byte[] RawIndividualVictory;
		public Byte[] RawDisables;
		public Byte[] RawRemaining;

		// NOTE: Unknown values
		public Int32 Unknown1;
		public Int32 Unknown2;
		public Int32 Unknown3;
		public Byte Unknown4;
		public Single Unknown5;

		public class ScnPlayerSettings
		{
			public const Int32 ExpectedUnknownInfo = 4;
			public const Int32 ExpectedUnknownResource = 0;

			public String Name;
			public Int32 NameStringID;
			public Boolean Active;
			public Boolean Human;
			public Int32 Civ;
			public Int32 UnknownInfo;

			public String FilenameAI;
			public String FilenameCTY;
			public String FilenamePER;
			public String ScriptAI;
			public String ScriptCTY;
			public String ScriptPER;
			public Byte AIType;

			public Int32 ResourceWood;
			public Int32 ResourceFood;
			public Int32 ResourceGold;
			public Int32 ResourceStone;
			public Int32 ResourceOre;
			public Int32 UnknownResource;

			public Int32[] Diplomacy;
			public Boolean AlliedVictory;
			public Int32 StartingAge;
		}

		public class ScnMessages
		{
			public String Instructions;
			public String Hints;
			public String Victory;
			public String Loss;
			public String History;
			public String Scouts;

			public Int32 InstructionsStringID;
			public Int32 HintsStringID;
			public Int32 VictoryStringID;
			public Int32 LossStringID;
			public Int32 HistoryStringID;
			public Int32 ScoutsStringID;

		}

		public class ScnCinematics
		{
			public const Int16 ExpectedUnknownBitmapTrue = -1;
			public const Int16 ExpectedUnknownBitmapFalse = 1;

			public String FilenamePregame;
			public String FilenameVictory;
			public String FilenameLoss;
			public String FilenameBitmap;

			public Int32 BitmapWidth;
			public Int32 BitmapHeight;
			public Int16 BitmapUnknown;

			public Byte[] RawBitmap;
		}

		public class ScnGlobalVictory
		{
			public Boolean RequireConquest;
			public Int32 Ruins;
			public Int32 Artifacts;
			public Int32 Discovery;
			public Int32 PercentExplored;
			public Boolean RequireAllCustom;
			public Int32 Mode;
			public Int32 Score;
			public Int32 Time;
			public Int32 Unknown;
		}

		public class ScnResourceCopy
		{
			public const Single ExpectedUnknownResource = 0;

			public Single ResourceGold;
			public Single ResourceWood;
			public Single ResourceFood;
			public Single ResourceStone;
			public Single ResourceOre;
			public Single UnknownResource;
			public Single PopulationLimit;
		}

		public class ScnMap
		{
			public Int32 CameraX;
			public Int32 CameraY;
			public Int32 GeneratorId;
			public Int32 SizeX;
			public Int32 SizeY;
			public Tile[] LinearTiles;

			public struct Tile
			{
				public const Byte ExpectedUnknown = 0;

				public Byte Id;
				public Byte Elevation;
				public Byte Unknown;

				public Tile(Byte id, Byte el, Byte un)
				{
					Id = id;
					Elevation = el;
					Unknown = un;
				}
			}
		}

		public class ScnUnit
		{
			public const Single ExpectedUnknown1 = 1;
			public const Byte ExpectedUnknown2 = 2;

			public Single PosX;
			public Single PosY;
			public Int32 Id;
			public Int16 UnitId;
			public Single Rotation;
			public Int16 InitialFrame;
			public Int32 GarrisonnedInId;
			public Single Unknown1;
			public Byte Unknown2;
		}
	}
}

